package com.leagmain.flutter.photoloader

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.leagmain.android.photoloader.Photo
import com.leagmain.android.photoloader.PhotoJsonAdapter
import com.leagmain.android.photoloader.loadPhotos
import com.leagmain.android.photoloader.loadPhotosAsJsonString
import com.squareup.moshi.Moshi
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

/** PhotoLoaderPlugin */
class PhotoloaderPlugin : FlutterPlugin, MethodCallHandler {
    private lateinit var channel: MethodChannel

    private lateinit var applicationContext: Context

    private var photos = emptyList<Photo>()
    private val moshi = Moshi.Builder().build()
    private val photoJsonAdapter = PhotoJsonAdapter(moshi)

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        applicationContext = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "photoloader")
        channel.setMethodCallHandler(this);
    }


    companion object {
        private const val ARG_ID = "id"
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "photoloader")
            channel.setMethodCallHandler(PhotoloaderPlugin())
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "loadPhotosAsJsonString" -> {
                result.success(applicationContext.contentResolver.loadPhotosAsJsonString())
            }
            "loadPhotos" -> {
                photos = applicationContext.contentResolver.loadPhotos()
                result.success(photos.map {
                    mapOf(
                            "id" to it.id,
                            "name" to it.displayName,
                            "path" to it.dataPath,
                            "title" to it.title
                    )
                })
            }
            "load" -> {
                photos = applicationContext.contentResolver.loadPhotos()
                result.success(true)
            }
            "getPhotoIds" -> {
                result.success(moshi.adapter(List::class.java).toJson(photos))
            }
            "getPhotoById" -> {
                if (call.hasArgument(ARG_ID)) {
                    result.error("arguments", "missing `id`", null)
                    return
                }
                val id = call.argument<String>(ARG_ID)
                val photo = photos.singleOrNull { it.id == id }

                if (photo == null) {
                    result.error("no photo", "no photo for id $id", null)
                    return
                }
                result.success(photoJsonAdapter.toJson(photo))
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}
